@@ -38,11 +38,15 @@
 using namespace Opcode;
 
 
-bool Opcode::InitOpcode()
+static OPCODE_AbortHandler g_fnAbortHandler = NULL;
+
+
+bool Opcode::InitOpcode(OPCODE_AbortHandler fnAbortHandler/*=NULL*/)
 {
 	//Log("// Initializing OPCODE\n\n");
 //	LogAPIInfo();
-	
+
+	g_fnAbortHandler = fnAbortHandler;
 	return true;
 }
 
@@ -65,3 +69,14 @@ void ModuleDetach()
 }
 
 #endif
+
+/*extern */
+void OPCODE_NORETURN IceAbort()
+{
+	if (g_fnAbortHandler != NULL)
+	{
+		g_fnAbortHandler();
+	}
+
+	abort();
+}
