@@ -1,3 +1,25 @@
+/*************************************************************************
+ *                                                                       *
+ * Open Dynamics Engine, Copyright (C) 2001,2002 Russell L. Smith.       *
+ * All rights reserved.  Email: russ@q12.org   Web: www.q12.org          *
+ *                                                                       *
+ * This library is free software; you can redistribute it and/or         *
+ * modify it under the terms of EITHER:                                  *
+ *   (1) The GNU Lesser General Public License as published by the Free  *
+ *       Software Foundation; either version 2.1 of the License, or (at  *
+ *       your option) any later version. The text of the GNU Lesser      *
+ *       General Public License is included with this library in the     *
+ *       file LICENSE.TXT.                                               *
+ *   (2) The BSD-style license that is included with this library in     *
+ *       the file LICENSE-BSD.TXT.                                       *
+ *                                                                       *
+ * This library is distributed in the hope that it will be useful,       *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files    *
+ * LICENSE.TXT and LICENSE-BSD.TXT for more details.                     *
+ *                                                                       *
+ *************************************************************************/
+
 #include <ode/ode.h>
 #include <drawstuff/drawstuff.h>
 #include "texturepath.h"
@@ -31,7 +53,7 @@ const dReal topmass = 1.0f;
 
 #define MAX_CONTACTS 4
 
-static void nearCallback (void *data, dGeomID o1, dGeomID o2)
+static void nearCallback (void *, dGeomID o1, dGeomID o2)
 {
     // for drawing the contact points
     dMatrix3 RI;
@@ -66,6 +88,9 @@ static void start()
   static float xyz[3] = {4.777f, -2.084f, 2.18f};
   static float hpr[3] = {153.0f, -14.5f, 0.0f};
   dsSetViewpoint (xyz,hpr);
+  printf ("Orange top approximates conservation of angular momentum\n");
+  printf ("Green top uses conservation of angular velocity\n");
+  printf ("---\n");
   printf ("SPACE to reset\n");
   printf ("A to tilt the tops.\n");
   printf ("T to toggle showing the contact points.\n");
@@ -156,8 +181,8 @@ static void reset()
     top1->setPosition(0.8f, -2, 2);
     top2->setPosition(0.8f, 2, 2);
     
-    top1->setAngularVel(0,0,5);
-    top2->setAngularVel(0,0,5);
+    top1->setAngularVel(1,0,7);
+    top2->setAngularVel(1,0,7);
     
     top1->setLinearVel(0,0.2f,0);
     top2->setLinearVel(0,0.2f,0);
