@@ -273,7 +273,7 @@ static void command (int cmd)
             dRFromAxisAndAngle (R,0,0,1,dRandReal()*10.0-5.0);
         }
         dBodySetRotation (obj[i].body,R);
-        dBodySetData (obj[i].body,(void*)(size_t)i);
+        dBodySetData (obj[i].body,(void*)(dsizeint)i);
 
         if (cmd == 'b') {
             dMassSetBox (&m,DENSITY,sides[0],sides[1],sides[2]);
@@ -307,6 +307,8 @@ static void command (int cmd)
             dTriMeshDataID new_tmdata = dGeomTriMeshDataCreate();
             dGeomTriMeshDataBuildSingle(new_tmdata, &Vertices[0], 3 * sizeof(float), VertexCount, 
                                         (dTriIndex*)&Indices[0], IndexCount, 3 * sizeof(dTriIndex));
+            dGeomTriMeshDataPreprocess2(new_tmdata, (1U << dTRIDATAPREPROCESS_BUILD_FACE_ANGLES), NULL);
+
 
             obj[i].geom[0] = dCreateTriMesh(space, new_tmdata, 0, 0, 0);
 
@@ -630,10 +632,13 @@ int main (int argc, char **argv)
   memset (obj,0,sizeof(obj));
 
   // note: can't share tridata if intending to trimesh-trimesh collide
+  const unsigned preprocessFlags = (1U << dTRIDATAPREPROCESS_BUILD_CONCAVE_EDGES) | (1U << dTRIDATAPREPROCESS_BUILD_FACE_ANGLES);
   TriData1 = dGeomTriMeshDataCreate();
   dGeomTriMeshDataBuildSingle(TriData1, &Vertices[0], 3 * sizeof(float), VertexCount, (dTriIndex*)&Indices[0], IndexCount, 3 * sizeof(dTriIndex));
+  dGeomTriMeshDataPreprocess2(TriData1, preprocessFlags, NULL);
   TriData2 = dGeomTriMeshDataCreate();
   dGeomTriMeshDataBuildSingle(TriData2, &Vertices[0], 3 * sizeof(float), VertexCount, (dTriIndex*)&Indices[0], IndexCount, 3 * sizeof(dTriIndex));
+  dGeomTriMeshDataPreprocess2(TriData2, preprocessFlags, NULL);
   
   TriMesh1 = dCreateTriMesh(space, TriData1, 0, 0, 0);
   TriMesh2 = dCreateTriMesh(space, TriData2, 0, 0, 0);
