@@ -53,7 +53,7 @@ dxConvex::dxConvex (dSpaceID space,
                     const dReal *_points,
                     unsigned int _pointcount,
                     const unsigned int *_polygons) :
-    dxGeom (space,1)
+dxGeom (space,1)
 {
     dAASSERT (_planes != NULL);
     dAASSERT (_points != NULL);
@@ -303,13 +303,6 @@ inline bool ClosestPointInRay(const dVector3 Origin1,
     return true;
 }
 
-/*! \brief Clamp n to lie within the range [min, max] */
-inline float Clamp(float n, float min, float max)
-{
-    if (n < min) return min;
-    if (n > max) return max;
-    return n;
-}
 /*! \brief Returns the Closest Points from Segment 1 to Segment 2
   \param p1 start of segment 1
   \param q1 end of segment 1
@@ -320,7 +313,7 @@ inline float Clamp(float n, float min, float max)
   \return true if there is a closest point, false if the rays are paralell.
   \note Adapted from Christer Ericson's Real Time Collision Detection Book.
 */
-inline float ClosestPointBetweenSegments(dVector3& p1,
+inline void ClosestPointBetweenSegments(dVector3& p1,
                                          dVector3& q1,
                                          dVector3& p2,
                                          dVector3& q2,
@@ -329,8 +322,8 @@ inline float ClosestPointBetweenSegments(dVector3& p1,
 {
     // s & t were originaly part of the output args, but since
     // we don't really need them, we'll just declare them in here
-    float s;
-    float t;
+    dReal s;
+    dReal t;
     dVector3 d1 = {q1[0] - p1[0],
         q1[1] - p1[1],
         q1[2] - p1[2]};
@@ -340,9 +333,9 @@ inline float ClosestPointBetweenSegments(dVector3& p1,
     dVector3 r  = {p1[0] - p2[0],
         p1[1] - p2[1],
         p1[2] - p2[2]};
-    float a = dCalcVectorDot3(d1, d1);
-    float e = dCalcVectorDot3(d2, d2);
-    float f = dCalcVectorDot3(d2, r);
+    dReal a = dCalcVectorDot3(d1, d1);
+    dReal e = dCalcVectorDot3(d2, d2);
+    dReal f = dCalcVectorDot3(d2, r);
     // Check if either or both segments degenerate into points
     if (a <= dEpsilon && e <= dEpsilon)
     {
@@ -350,37 +343,35 @@ inline float ClosestPointBetweenSegments(dVector3& p1,
         s = t = 0.0f;
         dVector3Copy(p1,c1);
         dVector3Copy(p2,c2);
-        return (c1[0] - c2[0])*(c1[0] - c2[0])+
-            (c1[1] - c2[1])*(c1[1] - c2[1])+
-            (c1[2] - c2[2])*(c1[2] - c2[2]);
+        return;
     }
     if (a <= dEpsilon)
     {
         // First segment degenerates into a point
         s = 0.0f;
         t = f / e; // s = 0 => t = (b*s + f) / e = f / e
-        t = Clamp(t, 0.0f, 1.0f);
+        t = dxClamp(t, 0.0f, 1.0f);
     }
     else
     {
-        float c = dCalcVectorDot3(d1, r);
+        dReal c = dCalcVectorDot3(d1, r);
         if (e <= dEpsilon)
         {
             // Second segment degenerates into a point
             t = 0.0f;
-            s = Clamp(-c / a, 0.0f, 1.0f); // t = 0 => s = (b*t - c) / a = -c / a
+            s = dxClamp(-c / a, 0.0f, 1.0f); // t = 0 => s = (b*t - c) / a = -c / a
         }
         else
         {
             // The general non degenerate case starts here
-            float b = dCalcVectorDot3(d1, d2);
-            float denom = a*e-b*b; // Always nonnegative
+            dReal b = dCalcVectorDot3(d1, d2);
+            dReal denom = a*e-b*b; // Always nonnegative
 
             // If segments not parallel, compute closest point on L1 to L2, and
             // clamp to segment S1. Else pick arbitrary s (here 0)
             if (denom != 0.0f)
             {
-                s = Clamp((b*f - c*e) / denom, 0.0f, 1.0f);
+                s = dxClamp((b*f - c*e) / denom, 0.0f, 1.0f);
             }
             else s = 0.0f;
 #if 0
@@ -393,22 +384,22 @@ inline float ClosestPointBetweenSegments(dVector3& p1,
             // and clamp s to [0, 1]
             if (t < 0.0f) {
                 t = 0.0f;
-                s = Clamp(-c / a, 0.0f, 1.0f);
+                s = dxClamp(-c / a, 0.0f, 1.0f);
             } else if (t > 1.0f) {
                 t = 1.0f;
-                s = Clamp((b - c) / a, 0.0f, 1.0f);
+                s = dxClamp((b - c) / a, 0.0f, 1.0f);
             }
 #else
-            float tnom = b*s + f;
+            dReal tnom = b*s + f;
             if (tnom < 0.0f)
             {
                 t = 0.0f;
-                s = Clamp(-c / a, 0.0f, 1.0f);
+                s = dxClamp(-c / a, 0.0f, 1.0f);
             }
             else if (tnom > e)
             {
                 t = 1.0f;
-                s = Clamp((b - c) / a, 0.0f, 1.0f);
+                s = dxClamp((b - c) / a, 0.0f, 1.0f);
             }
             else
             {
@@ -424,19 +415,16 @@ inline float ClosestPointBetweenSegments(dVector3& p1,
     c2[0] = p2[0] + d2[0] * t;
     c2[1] = p2[1] + d2[1] * t;
     c2[2] = p2[2] + d2[2] * t;
-    return (c1[0] - c2[0])*(c1[0] - c2[0])+
-        (c1[1] - c2[1])*(c1[1] - c2[1])+
-        (c1[2] - c2[2])*(c1[2] - c2[2]);
 }
 
 #if 0
-float tnom = b*s + f;
+dReal tnom = b*s + f;
 if (tnom < 0.0f) {
     t = 0.0f;
-    s = Clamp(-c / a, 0.0f, 1.0f);
+    s = dxClamp(-c / a, 0.0f, 1.0f);
 } else if (tnom > e) {
     t = 1.0f;
-    s = Clamp((b - c) / a, 0.0f, 1.0f);
+    s = dxClamp((b - c) / a, 0.0f, 1.0f);
 } else {
     t = tnom / e;
 }
@@ -509,7 +497,7 @@ inline bool IsPointInConvex(dVector3 p,
 */
 inline bool IsPointInPolygon(dVector3 p,
                              const unsigned int *polygon,
-			                 dReal *plane,
+                             dReal *plane,
                              dxConvex *convex,
                              dVector3 out)
 {
@@ -534,7 +522,7 @@ inline bool IsPointInPolygon(dVector3 p,
     b[0]=convex->final_posr->pos[0]+b[0];
     b[1]=convex->final_posr->pos[1]+b[1];
     b[2]=convex->final_posr->pos[2]+b[2];
-    
+
     for(unsigned i=0; i != pointcount; ++i)
     {
         a[0] = b[0];
@@ -559,7 +547,7 @@ inline bool IsPointInPolygon(dVector3 p,
         {
             dReal ab_m2 = dCalcVectorDot3(ab, ab);
             dReal s = ab_m2 != REAL(0.0) ? dCalcVectorDot3(ab, ap) / ab_m2 : REAL(0.0);
-            
+
             if (s <= REAL(0.0))
             {
                 out[0] = a[0];
@@ -858,7 +846,7 @@ bool CheckEdgeIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,int& curc,
         e2[1]+=cvx1.final_posr->pos[1];
         e2[2]+=cvx1.final_posr->pos[2];
         const unsigned int* pPoly=cvx2.polygons;
-        for(size_t j=0;j<cvx2.planecount;++j)
+        for(sizeint j=0;j<cvx2.planecount;++j)
         {
             // Rotate
             dMultiply0_331(plane,cvx2.final_posr->R,cvx2.planes+(j*4));
@@ -877,7 +865,7 @@ bool CheckEdgeIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,int& curc,
                 if(IsPointInPolygon(target->pos,pPoly,plane,&cvx2,q))
                 {
                     target->depth = dInfinity;
-                    for(size_t k=0;k<cvx2.planecount;++k)
+                    for(sizeint k=0;k<cvx2.planecount;++k)
                     {
                         if(k==j) continue; // we're already at 0 depth on this plane
                         // Rotate
@@ -924,7 +912,7 @@ struct ConvexConvexSATOutput
   \param min_depth [IN/OUT] Used to input as well as output the minimum depth so far, must be set to a huge value such as dInfinity for initialization.
   \param g1 [OUT] Pointer to the convex which should be used in the returned contact as g1
   \param g2 [OUT] Pointer to the convex which should be used in the returned contact as g2
- */
+*/
 inline bool CheckSATConvexFaces(dxConvex& cvx1,
                                 dxConvex& cvx2,
                                 ConvexConvexSATOutput& ccso)
@@ -971,7 +959,7 @@ inline bool CheckSATConvexFaces(dxConvex& cvx1,
   \param min_depth [IN/OUT] Used to input as well as output the minimum depth so far, must be set to a huge value such as dInfinity for initialization.
   \param g1 [OUT] Pointer to the convex which should be used in the returned contact as g1
   \param g2 [OUT] Pointer to the convex which should be used in the returned contact as g2
- */
+*/
 inline bool CheckSATConvexEdges(dxConvex& cvx1,
                                 dxConvex& cvx2,
                                 ConvexConvexSATOutput& ccso)
@@ -1108,9 +1096,7 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
     ccso.min_depth=dInfinity; // Min not min at all
     ccso.depth_type=0; // no type
     // precompute distance vector
-    ccso.dist[0] = cvx2.final_posr->pos[0]-cvx1.final_posr->pos[0];
-    ccso.dist[1] = cvx2.final_posr->pos[1]-cvx1.final_posr->pos[1];
-    ccso.dist[2] = cvx2.final_posr->pos[2]-cvx1.final_posr->pos[2];
+    dSubtractVectors3(ccso.dist, cvx2.final_posr->pos, cvx1.final_posr->pos);
     int maxc = flags & NUMC_MASK;
     dIASSERT(maxc != 0);
     dVector3 i1,i2,r1,r2; // edges of incident and reference faces respectively
@@ -1146,9 +1132,7 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
             bool outside,out;
             dVector3Copy(ccso.dist,dist);
             reference_side = GetSupportSide(dist,cvx1);
-            dist[0]=-dist[0];
-            dist[1]=-dist[1];
-            dist[2]=-dist[2];
+            dNegateVector3(dist);
             incident_side = GetSupportSide(dist,cvx2);
 
             pReferencePoly = cvx1.polygons;
@@ -1211,22 +1195,31 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                         plane[3];
                     if(d1*d2<0)
                     {
+                        out = false;
+
                         // Edge intersects plane
-                        IntersectSegmentPlane(r1,r2,plane,t,p);
-                        // Check the resulting point again to make sure it is inside the reference convex
-                        out=false;
-                        for(unsigned int k=0;k<cvx1.planecount;++k)
+                        if (!IntersectSegmentPlane(r1,r2,plane,t,p))
                         {
-                            d = p[0]*cvx1.planes[(k*4)+0]+
-                                p[1]*cvx1.planes[(k*4)+1]+
-                                p[2]*cvx1.planes[(k*4)+2]-
-                                cvx1.planes[(k*4)+3];
-                            if(d>0)
-                            {
-                                out = true;
-                                break;
-                            };
+                            out = true;
                         }
+
+                        if (!out)
+                        {
+                            // Check the resulting point again to make sure it is inside the reference convex
+                            for (unsigned int k = 0; k < cvx1.planecount; ++k)
+                            {
+                                d = p[0]*cvx1.planes[(k*4)+0]+
+                                    p[1]*cvx1.planes[(k*4)+1]+
+                                    p[2]*cvx1.planes[(k*4)+2]-
+                                    cvx1.planes[(k*4)+3];
+                                if(d>0)
+                                {
+                                    out = true;
+                                    break;
+                                }
+                            }
+                        }
+
                         if(!out)
                         {
 #if 0
@@ -1249,11 +1242,12 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                                 rplane[3];
                             if(d>0)
                             {
-                                dVector3Copy(p,SAFECONTACT(flags, contact, contacts, skip)->pos);
-                                dVector3Copy(rplane,SAFECONTACT(flags, contact, contacts, skip)->normal);
-                                SAFECONTACT(flags, contact, contacts, skip)->g1=&cvx1;
-                                SAFECONTACT(flags, contact, contacts, skip)->g2=&cvx2;
-                                SAFECONTACT(flags, contact, contacts, skip)->depth=d;
+                                dContactGeom *target = SAFECONTACT(flags, contact, contacts, skip);
+                                dVector3Copy(p, target->pos);
+                                dVector3Copy(rplane, target->normal);
+                                target->g1 = &cvx1;
+                                target->g2 = &cvx2;
+                                target->depth = d;
                                 ++contacts;
                                 if (contacts==maxc) return contacts;
                             }
@@ -1271,11 +1265,12 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                     rplane[3];
                 if(d>0)
                 {
-                    dVector3Copy(i1,SAFECONTACT(flags, contact, contacts, skip)->pos);
-                    dVector3Copy(rplane,SAFECONTACT(flags, contact, contacts, skip)->normal);
-                    SAFECONTACT(flags, contact, contacts, skip)->g1=&cvx1;
-                    SAFECONTACT(flags, contact, contacts, skip)->g2=&cvx2;
-                    SAFECONTACT(flags, contact, contacts, skip)->depth=d;
+                    dContactGeom *target = SAFECONTACT(flags, contact, contacts, skip);
+                    dVector3Copy(i1, target->pos);
+                    dVector3Copy(rplane, target->normal);
+                    target->g1 = &cvx1;
+                    target->g2 = &cvx2;
+                    target->depth = d;
                     ++contacts;
                     if (contacts==maxc) return contacts;
                 }
@@ -1335,9 +1330,10 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                     outside = false;
                     for(int j=0;j<contacts;++j)
                     {
-                        if((SAFECONTACT(flags, contact, j, skip)->pos[0]==i1[0])&&
-                            (SAFECONTACT(flags, contact, j, skip)->pos[1]==i1[1])&&
-                            (SAFECONTACT(flags, contact, j, skip)->pos[2]==i1[2]))
+                        dContactGeom *cur_contact = SAFECONTACT(flags, contact, j, skip);
+                        if((cur_contact->pos[0] == i1[0]) &&
+                            (cur_contact->pos[1] == i1[1]) &&
+                            (cur_contact->pos[2] == i1[2]))
                         {
                             outside=true;
                         }
@@ -1350,11 +1346,12 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                             rplane[3];
                         if(d>0)
                         {
-                            dVector3Copy(i1,SAFECONTACT(flags, contact, contacts, skip)->pos);
-                            dVector3Copy(rplane,SAFECONTACT(flags, contact, contacts, skip)->normal);
-                            SAFECONTACT(flags, contact, contacts, skip)->g1=&cvx1;
-                            SAFECONTACT(flags, contact, contacts, skip)->g2=&cvx2;
-                            SAFECONTACT(flags, contact, contacts, skip)->depth=d;
+                            dContactGeom *target = SAFECONTACT(flags, contact, contacts, skip);
+                            dVector3Copy(i1, target->pos);
+                            dVector3Copy(rplane, target->normal);
+                            target->g1 = &cvx1;
+                            target->g2 = &cvx2;
+                            target->depth = d;
                             ++contacts;
                             if (contacts==maxc) return contacts;
                         }
@@ -1362,19 +1359,36 @@ int TestConvexIntersection(dxConvex& cvx1,dxConvex& cvx2, int flags,
                 }
             }
         }
-        else if(ccso.depth_type==2) // edge-edge
+        else if (ccso.depth_type == 2) // edge-edge
         {
-            dVector3 c1,c2;
-            //float s,t;
-            SAFECONTACT(flags, contact, contacts, skip)->depth = 
-                dSqrt(ClosestPointBetweenSegments(ccso.e1a,ccso.e1b,ccso.e2a,ccso.e2b,c1,c2));
-            SAFECONTACT(flags, contact, contacts, skip)->g1=&cvx1;
-            SAFECONTACT(flags, contact, contacts, skip)->g2=&cvx2;
-            dVector3Copy(c1,SAFECONTACT(flags, contact, contacts, skip)->pos);
-            SAFECONTACT(flags, contact, contacts, skip)->normal[0] = c2[0]-c1[0];
-            SAFECONTACT(flags, contact, contacts, skip)->normal[1] = c2[1]-c1[1];
-            SAFECONTACT(flags, contact, contacts, skip)->normal[2] = c2[2]-c1[2];    
-            dNormalize3(SAFECONTACT(flags, contact, contacts, skip)->normal);
+            dVector3 c1, c2;
+            ClosestPointBetweenSegments(ccso.e1a, ccso.e1b, ccso.e2a, ccso.e2b, c1, c2);
+
+            dContactGeom *target = SAFECONTACT(flags, contact, contacts, skip);
+            dSubtractVectors3(target->normal, c2, c1);
+            dReal depth_square = dCalcVectorLengthSquare3(target->normal);
+
+            if (dxSafeNormalize3(target->normal))
+            {
+                target->depth = dSqrt(depth_square);
+            }
+            else
+            {
+                // If edges coincide return direction from one center to the other as the contact normal
+                dVector3Copy(ccso.dist, target->normal);
+
+                if (!dxSafeNormalize3(target->normal))
+                {
+                    // If the both centers coincide as well return an arbitrary vector. The depth is going to be zero anyway.
+                    dAssignVector3(target->normal, 1, 0, 0);
+                }
+
+                target->depth = 0; // Since the edges coincide, return a contact of zero depth
+            }
+
+            target->g1 = &cvx1;
+            target->g2 = &cvx2;
+            dVector3Copy(c1, target->pos);
             contacts++;
         }
         return contacts;
@@ -1457,16 +1471,16 @@ int dCollideRayConvex (dxGeom *o1, dxGeom *o2, int flags,
 }
 #else
 // Ray - Convex collider by David Walters, June 2006
-int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
-                      int flags, dContactGeom *contact, int skip )
+int dCollideRayConvex(dxGeom *o1, dxGeom *o2,
+                      int flags, dContactGeom *contact, int skip)
 {
-    dIASSERT( skip >= (int)sizeof(dContactGeom) );
-    dIASSERT( o1->type == dRayClass );
-    dIASSERT( o2->type == dConvexClass );
-    dIASSERT ((flags & NUMC_MASK) >= 1);
+    dIASSERT(skip >= (int)sizeof(dContactGeom));
+    dIASSERT(o1->type == dRayClass);
+    dIASSERT(o2->type == dConvexClass);
+    dIASSERT((flags & NUMC_MASK) >= 1);
 
-    dxRay* ray = (dxRay*) o1;
-    dxConvex* convex = (dxConvex*) o2;
+    dxRay* ray = (dxRay*)o1;
+    dxConvex* convex = (dxConvex*)o2;
 
     contact->g1 = ray;
     contact->g2 = convex;
@@ -1474,25 +1488,38 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
     contact->side2 = -1; // TODO: set plane index?
 
     dReal alpha, beta, nsign;
-    int flag;
+    int flag = 0;
 
     //
     // Compute some useful info
     //
 
-    flag = 0;	// Assume start point is behind all planes.
+    dVector3 ray_pos = {
+        ray->final_posr->pos[0] - convex->final_posr->pos[0],
+        ray->final_posr->pos[1] - convex->final_posr->pos[1],
+        ray->final_posr->pos[2] - convex->final_posr->pos[2]
+    };
 
-    for ( unsigned int i = 0; i < convex->planecount; ++i )
+    dVector3 ray_dir = {
+        ray->final_posr->R[0 * 4 + 2],
+        ray->final_posr->R[1 * 4 + 2],
+        ray->final_posr->R[2 * 4 + 2]
+    };
+
+    dMultiply1_331(ray_pos, convex->final_posr->R, ray_pos);
+    dMultiply1_331(ray_dir, convex->final_posr->R, ray_dir);
+
+    for (unsigned int i = 0; i < convex->planecount; ++i)
     {
         // Alias this plane.
-        const dReal* plane = convex->planes + ( i * 4 );
+        const dReal* plane = convex->planes + (i * 4);
 
         // If alpha >= 0 then start point is outside of plane.
-        alpha = dCalcVectorDot3( plane, ray->final_posr->pos ) - plane[3];
+        alpha = dCalcVectorDot3(plane, ray_pos) - plane[3];
 
         // If any alpha is positive, then
         // the ray start is _outside_ of the hull
-        if ( alpha >= 0 )
+        if (alpha >= 0)
         {
             flag = 1;
             break;
@@ -1500,7 +1527,7 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
     }
 
     // If the ray starts inside the convex hull, then everything is flipped.
-    nsign = ( flag ) ? REAL( 1.0 ) : REAL( -1.0 );
+    nsign = (flag) ? REAL(1.0) : REAL(-1.0);
 
 
     //
@@ -1510,47 +1537,47 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
     // Assume no contacts.
     contact->depth = dInfinity;
 
-    for ( unsigned int i = 0; i < convex->planecount; ++i )
+    for (unsigned int i = 0; i < convex->planecount; ++i)
     {
         // Alias this plane.
-        const dReal* plane = convex->planes + ( i * 4 );
+        const dReal* plane = convex->planes + (i * 4);
 
         // If alpha >= 0 then point is outside of plane.
-        alpha = nsign * ( dCalcVectorDot3( plane, ray->final_posr->pos ) - plane[3] );
+        alpha = nsign * (dCalcVectorDot3(plane, ray_pos) - plane[3]);
 
         // Compute [ plane-normal DOT ray-normal ], (/flip)
-        beta = dCalcVectorDot3_13( plane, ray->final_posr->R+2 ) * nsign;
+        beta = dCalcVectorDot3(plane, ray_dir) * nsign;
 
         // Ray is pointing at the plane? ( beta < 0 )
         // Ray start to plane is within maximum ray length?
         // Ray start to plane is closer than the current best distance?
-        if ( beta < -dEpsilon &&
+        if (beta < -dEpsilon &&
             alpha >= 0 && alpha <= ray->length &&
-            alpha < contact->depth )
+            alpha < contact->depth)
         {
             // Compute contact point on convex hull surface.
-            contact->pos[0] = ray->final_posr->pos[0] + alpha * ray->final_posr->R[0*4+2];
-            contact->pos[1] = ray->final_posr->pos[1] + alpha * ray->final_posr->R[1*4+2];
-            contact->pos[2] = ray->final_posr->pos[2] + alpha * ray->final_posr->R[2*4+2];
+            contact->pos[0] = ray_pos[0] + alpha * ray_dir[0];
+            contact->pos[1] = ray_pos[1] + alpha * ray_dir[1];
+            contact->pos[2] = ray_pos[2] + alpha * ray_dir[2];
 
             flag = 0;
 
             // For all _other_ planes.
-            for ( unsigned int j = 0; j < convex->planecount; ++j )
+            for (unsigned int j = 0; j < convex->planecount; ++j)
             {
-                if ( i == j )
+                if (i == j)
                     continue;	// Skip self.
 
                 // Alias this plane.
-                const dReal* planej = convex->planes + ( j * 4 );
+                const dReal* planej = convex->planes + (j * 4);
 
                 // If beta >= 0 then start is outside of plane.
-                beta = dCalcVectorDot3( planej, contact->pos ) - plane[3];
+                beta = dCalcVectorDot3(planej, contact->pos) - planej[3];
 
                 // If any beta is positive, then the contact point
                 // is not on the surface of the convex hull - it's just
                 // intersecting some part of its infinite extent.
-                if ( beta > dEpsilon )
+                if (beta > dEpsilon)
                 {
                     flag = 1;
                     break;
@@ -1558,7 +1585,7 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
             }
 
             // Contact point isn't outside hull's surface? then it's a good contact!
-            if ( flag == 0 )
+            if (flag == 0)
             {
                 // Store the contact normal, possibly flipped.
                 contact->normal[0] = nsign * plane[0];
@@ -1568,7 +1595,7 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
                 // Store depth
                 contact->depth = alpha;
 
-                if ((flags & CONTACTS_UNIMPORTANT) && contact->depth <= ray->length )
+                if ((flags & CONTACTS_UNIMPORTANT) && contact->depth <= ray->length)
                 {
                     // Break on any contact if contacts are not important
                     break;
@@ -1577,7 +1604,17 @@ int dCollideRayConvex( dxGeom *o1, dxGeom *o2,
         }
     }
     // Contact?
-    return ( contact->depth <= ray->length );
+    if (contact->depth <= ray->length)
+    {
+        // Adjust contact position and normal back to global space
+        dMultiply0_331(contact->pos, convex->final_posr->R, contact->pos);
+        dMultiply0_331(contact->normal, convex->final_posr->R, contact->normal);
+        contact->pos[0] += convex->final_posr->pos[0];
+        contact->pos[1] += convex->final_posr->pos[1];
+        contact->pos[2] += convex->final_posr->pos[2];
+        return true;
+    }
+    return false;
 }
 
 #endif
