@@ -4,7 +4,7 @@
  * All rights reserved.  Email: russ@q12.org   Web: www.q12.org          *
  *                                                                       *
  * Threading POSIX implementation file.                                  *
- * Copyright (C) 2011-2012 Oleh Derevenko. All rights reserved.          *
+ * Copyright (C) 2011-2019 Oleh Derevenko. All rights reserved.          *
  * e-mail: odar@eleks.com (change all "a" to "e")                        *
  *                                                                       *
  * This library is free software; you can redistribute it and/or         *
@@ -43,6 +43,7 @@
 #include "threading_fake_sync.h"
 #include "threading_atomics_provs.h"
 
+
 #if dBUILTIN_THREADING_IMPL_ENABLED
 
 #include <pthread.h>
@@ -54,10 +55,64 @@
 #endif
 
 
-#endif // #if dBUILTIN_THREADING_IMPL_ENABLED
+#if defined(__APPLE__)
+
+#if HAVE_GETTIMEOFDAY
+
+#include <sys/time.h>
+
+#if !defined(CLOCK_MONOTONIC)
+#define CLOCK_MONOTONIC 2
+#endif
+
+static inline 
+int _condvar_clock_gettime(int clock_type, timespec *ts)
+{
+    (void)clock_type; // Unused
+    timeval tv;
+    return gettimeofday(&tv, NULL) == 0 ? (ts->tv_sec = tv.tv_sec, ts->tv_nsec = tv.tv_usec * 1000, 0) : (-1);
+}
 
 
-#if dBUILTIN_THREADING_IMPL_ENABLED
+#else // #if !HAVE_GETTIMEOFDAY
+
+#error It is necessary to check manuals for the correct way of getting condvar wait time for this Apple system
+
+
+#endif // #if !HAVE_GETTIMEOFDAY
+
+
+#else // #if !defined(__APPLE__)
+
+#if !HAVE_PTHREAD_CONDATTR_SETCLOCK && !HAVE_NO_PTHREAD_CONDATTR_SETCLOCK
+
+// The code must be compiled without autoconf run, having the project generated by other means.
+// Assume the pthread_condattr_setclock() is available as it is true in most cases and it is the best we can do in such cases.
+#define HAVE_PTHREAD_CONDATTR_SETCLOCK 1
+
+
+#endif // #if !HAVE_PTHREAD_CONDATTR_SETCLOCK && !HAVE_NO_PTHREAD_CONDATTR_SETCLOCK
+
+
+#if HAVE_PTHREAD_CONDATTR_SETCLOCK
+
+static inline 
+int _condvar_clock_gettime(int clock_type, timespec *ts)
+{
+    return clock_gettime(clock_type, ts);
+}
+
+
+#else // #if !HAVE_PTHREAD_CONDATTR_SETCLOCK
+
+#error It is necessary to check manuals for the correct way of getting condvar wait time for this system
+
+
+#endif // #if !HAVE_PTHREAD_CONDATTR_SETCLOCK
+
+
+#endif // #if !defined(__APPLE__)
+
 
 /************************************************************************/
 /* dxCondvarWakeup class implementation                                 */
@@ -142,12 +197,14 @@ bool dxCondvarWakeup::DoInitializeObject()
 
         condattr_initialized = true;
 
+#if HAVE_PTHREAD_CONDATTR_SETCLOCK
         int condattr_clock_result = pthread_condattr_setclock(&cond_condattr, CLOCK_MONOTONIC);
         if (condattr_clock_result != EOK)
         {
             errno = condattr_clock_result;
             break;
         }
+#endif // #if HAVE_PTHREAD_CONDATTR_SETCLOCK
 
         int cond_result = pthread_cond_init(&m_wakeup_cond, &cond_condattr);
         if (cond_result != EOK)
@@ -306,7 +363,7 @@ bool dxCondvarWakeup::BlockAsAWaiter(const dThreadedWaitTime *timeout_time_ptr)
     {
         timespec current_time;
 
-        int clock_result = clock_gettime(CLOCK_MONOTONIC, &current_time);
+        int clock_result = _condvar_clock_gettime(CLOCK_MONOTONIC, &current_time);
         dICHECK(clock_result != -1);
 
         time_t wakeup_sec = current_time.tv_sec + timeout_time_ptr->wait_sec;
